@@ -1,28 +1,21 @@
 // SPDX-License-Identifier: MIT
+// JPEG XL format decoder.
 // Copyright (C) 2021 Artem Senichev <artemsen@gmail.com>
 
-//
-// JPEG XL image format support
-//
+#include "loader.h"
 
-#include "common.h"
-
-#include <cairo/cairo.h>
 #include <jxl/decode.h>
-#include <stdint.h>
-#include <stdio.h>
 #include <string.h>
 
 // JPEG XL loader implementation
-cairo_surface_t* load_jxl(const uint8_t* data, size_t size, char* format,
-                          size_t format_sz)
+enum loader_status decode_jxl(struct image* ctx, const uint8_t* data,
+                              size_t size)
 {
-    cairo_surface_t* surface = NULL;
-    uint8_t* buffer = NULL;
-    size_t buffer_sz;
     JxlDecoder* jxl;
     JxlBasicInfo info;
     JxlDecoderStatus status;
+    size_t buffer_sz;
+    argb_t* buffer = NULL;
 
     const JxlPixelFormat jxl_format = { .num_channels = 4, // ARBG
                                         .data_type = JXL_TYPE_UINT8,
@@ -33,7 +26,7 @@ cairo_surface_t* load_jxl(const uint8_t* data, size_t size, char* format,
     switch (JxlSignatureCheck(data, size)) {
         case JXL_SIG_NOT_ENOUGH_BYTES:
         case JXL_SIG_INVALID:
-            return NULL;
+            return ldr_unsupported;
         default:
             break;
     }
@@ -41,21 +34,21 @@ cairo_surface_t* load_jxl(const uint8_t* data, size_t size, char* format,
     // initialize decoder
     jxl = JxlDecoderCreate(NULL);
     if (!jxl) {
-        fprintf(stderr, "Unable to create JPEG XL decoder\n");
-        return NULL;
+        image_error(ctx, "unable to create jpeg xl decoder");
+        return ldr_fmterror;
     }
     status = JxlDecoderSetInput(jxl, data, size);
     if (status != JXL_DEC_SUCCESS) {
-        fprintf(stderr, "Unable to set JPEG XL buffer [%i]\n", status);
-        goto error;
+        image_error(ctx, "unable to set jpeg xl buffer: error %d", status);
+        goto fail;
     }
 
     // process decoding
     status =
         JxlDecoderSubscribeEvents(jxl, JXL_DEC_BASIC_INFO | JXL_DEC_FULL_IMAGE);
     if (status != JXL_DEC_SUCCESS) {
-        fprintf(stderr, "JPEG XL event subscription failed\n");
-        goto error;
+        image_error(ctx, "jpeg xl event subscription failed");
+        goto fail;
     }
     do {
         JxlDecoderStatus rc;
@@ -64,13 +57,14 @@ cairo_surface_t* load_jxl(const uint8_t* data, size_t size, char* format,
             case JXL_DEC_SUCCESS:
                 break; // decoding complete
             case JXL_DEC_ERROR:
-                fprintf(stderr, "JPEG XL decoder failed\n");
-                goto error;
+                image_error(ctx, "failed to decode jpeg xl");
+                goto fail;
             case JXL_DEC_BASIC_INFO:
                 rc = JxlDecoderGetBasicInfo(jxl, &info);
                 if (rc != JXL_DEC_SUCCESS) {
-                    fprintf(stderr, "Unable to get JPEG XL info [%i]\n", rc);
-                    goto error;
+                    image_error(ctx, "unable to get jpeg xl info: error %d",
+                                rc);
+                    goto fail;
                 }
                 break;
             case JXL_DEC_FULL_IMAGE:
@@ -79,61 +73,49 @@ cairo_surface_t* load_jxl(const uint8_t* data, size_t size, char* format,
                 // get image buffer size
                 rc = JxlDecoderImageOutBufferSize(jxl, &jxl_format, &buffer_sz);
                 if (rc != JXL_DEC_SUCCESS) {
-                    fprintf(stderr, "Unable to get JPEG XL buffer size [%i]\n",
-                            rc);
-                    goto error;
+                    image_error(ctx, "unable to get jpeg xl buffer: error %d",
+                                rc);
+                    goto fail;
                 }
-                // create image surface
-                surface = create_surface(info.xsize, info.ysize, true);
-                if (!surface) {
-                    goto error;
+                buffer = image_allocate(ctx, info.xsize, info.ysize);
+                if (!buffer) {
+                    goto fail;
                 }
                 // check buffer format
-                buffer = cairo_image_surface_get_data(surface);
-                if (buffer_sz !=
-                    info.ysize * cairo_image_surface_get_stride(surface)) {
-                    fprintf(stderr, "Unsupported JPEG XL buffer format\n");
-                    goto error;
+                if (buffer_sz != ctx->width * ctx->height * sizeof(argb_t)) {
+                    image_error(ctx, "unsupported jpeg xl buffer format");
+                    goto fail;
                 }
                 // set output buffer
                 rc = JxlDecoderSetImageOutBuffer(jxl, &jxl_format, buffer,
                                                  buffer_sz);
                 if (rc != JXL_DEC_SUCCESS) {
-                    fprintf(stderr, "Unable to set JPEG XL buffer [%i]\n", rc);
-                    goto error;
+                    image_error(ctx, "unable to set jpeg xl buffer: error %d",
+                                rc);
+                    goto fail;
                 }
                 break;
             default:
-                fprintf(stderr, "Unexpected JPEG XL status: %i\n", status);
+                image_error(ctx, "unexpected jpeg xl status %d", status);
         }
     } while (status != JXL_DEC_SUCCESS);
 
-    if (!buffer) {
-        // JXL_DEC_NEED_IMAGE_OUT_BUFFER was not handled, something went wrong
-        fprintf(stderr, "Missed buffer initialization in JPEG XL decoder\n");
-        goto error;
+    // convert ABGR -> ARGB
+    for (size_t i = 0; i < ctx->width * ctx->height; ++i) {
+        buffer[i] = ARGB_FROM_ABGR(buffer[i]);
     }
 
-    // convert colors: JPEG XL -> Cairo (RGBA -> ARGB)
-    for (size_t i = 0; i < info.xsize * info.ysize; ++i) {
-        uint8_t* pixel = buffer + i * jxl_format.num_channels;
-        const uint8_t tmp = pixel[0];
-        pixel[0] = pixel[2];
-        pixel[2] = tmp;
-    }
-    cairo_surface_mark_dirty(surface);
-
     // format description: total number of bits per pixel
-    snprintf(format, format_sz, "JPEG XL %ubit",
-             info.bits_per_sample * info.num_color_channels + info.alpha_bits);
+    image_add_meta(ctx, "Format", "JPEG XL %ubpp",
+                   info.bits_per_sample * info.num_color_channels +
+                       info.alpha_bits);
+    ctx->alpha = true;
 
     JxlDecoderDestroy(jxl);
-    return surface;
+    return ldr_success;
 
-error:
+fail:
     JxlDecoderDestroy(jxl);
-    if (surface) {
-        cairo_surface_destroy(surface);
-    }
-    return NULL;
+    image_deallocate(ctx);
+    return ldr_fmterror;
 }
