@@ -1,31 +1,30 @@
 // SPDX-License-Identifier: MIT
+// Wayland window.
 // Copyright (C) 2020 Artem Senichev <artemsen@gmail.com>
 
 #pragma once
 
-#include <cairo/cairo.h>
-#include <stdbool.h>
-#include <stddef.h>
-#include <stdint.h>
+#include "types.h"
+
 #include <xkbcommon/xkbcommon.h>
 
 /** UI event handlers. */
-struct handlers {
+struct wnd_handlers {
     /**
      * Redraw handler.
-     * @param[in] window surface to draw on
+     * @param data pointer to pixel data
      */
-    void (*on_redraw)(cairo_surface_t* window);
+    void (*on_redraw)(argb_t* data);
 
     /**
      * Window resize handler.
-     * @param[in] window surface to draw on
+     * @param width,height window size in pixels
      */
-    void (*on_resize)(void);
+    void (*on_resize)(size_t width, size_t height);
 
     /**
      * Key press handler.
-     * @param[in] key code of key pressed
+     * @param key code of key pressed
      * @return true if state was changed and window must be redrawn
      */
     bool (*on_keyboard)(xkb_keysym_t key);
@@ -33,14 +32,14 @@ struct handlers {
 
 /**
  * Create window.
- * @param[in] handlers event handlers
- * @param[in] width window width
- * @param[in] height window height
- * @param[in] app_id application id
+ * @param handlers event handlers
+ * @param width window width
+ * @param height window height
+ * @param app_id application id
  * @return true if operation completed successfully
  */
-bool create_window(const struct handlers* handlers, size_t width, size_t height,
-                   const char* app_id);
+bool create_window(const struct wnd_handlers* handlers, size_t width,
+                   size_t height, const char* app_id);
 
 /**
  * Show window and run event handler loop.
@@ -57,26 +56,14 @@ void destroy_window(void);
  */
 void close_window(void);
 
-/**
- * Get window width.
- * @return window width
- */
-size_t get_window_width(void);
-
-/**
- * Get window height.
- * @return window height
- */
-size_t get_window_height(void);
-
 /**
  * Set window title.
- * @param[in] title new window title
+ * @param file name of the displayed file for window title
  */
-void set_window_title(const char* title);
+void set_window_title(const char* file);
 
 /**
  * Enable or disable full screen mode.
- * @param[in] enable mode
+ * @param enable mode
  */
 void enable_fullscreen(bool enable);
